@@ -1,7 +1,9 @@
 #include <stdlib.h>
+#include <glib.h>
 #include <libosso.h>
 
 #include "appdata.h"
+#include "helpers.h"
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
 #endif
@@ -29,6 +31,11 @@ app_data_t* app_init() {
 	}
     myapp->actioninprog = NONE;
     myapp->zoomlevel = FIT_SCREEN;
+    myapp->showimgparam = FALSE;
+
+    /* Load configuration from GConf */
+    load_configuration(myapp);
+
 	/* Initialize OSSO context */
     myapp->ossocont = osso_initialize(PACKAGE,VERSION,TRUE,NULL);
 	if (myapp->ossocont == NULL) {
