@@ -6,6 +6,23 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkpixbuf.h>
 
+#include "imgparam-window.h"
+
+/*
+ * Constants definitions
+ */
+#define MAXDIRLEN 512
+#define MAXFNLEN 128
+#define MINBUTPRESSTIME 1
+#define MAXBUTPRESSDISTANCE 10
+#define MCE_SIGNAL_MATCH "type='signal'," \
+						 "sender='"    MCE_SERVICE     "'," \
+						 "path='"      MCE_SIGNAL_PATH "'," \
+						 "interface='" MCE_SIGNAL_IF   "'"
+#define APP_DBUS_SERVICE "com.nokia." PACKAGE
+#define APP_DBUS_OBJECT_PATH "com/nokia/" PACKAGE
+#define APP_DBUS_INTERFACE "com.nokia." PACKAGE
+
 /*
  * Program structure definition
  */
@@ -21,7 +38,7 @@ struct app_data {
 	HildonWindow* mainwin;
 	GtkWidget* image;
 	GdkPixbuf* imgbuffer;
-	GtkWidget* imgparamwin;
+	ImgparamWindow* imgparamwin;
 	gchar* imgparam;
 	gchar* currfname;
 	gchar* currdir;
@@ -33,6 +50,7 @@ struct app_data {
 	gdouble pressbuttony;
 	action_t actioninprog;
 	zoom_t zoomlevel;
+	gboolean showimgparam;
 };
 
 /*
