@@ -15,6 +15,7 @@
 #include <dbus/dbus.h>
 #include <mce/mode-names.h>
 #include <mce/dbus-names.h>
+#include <libosso.h>
 #include <hildon/hildon.h>
 #include <hildon/hildon-fm.h>
 #include <hildon/hildon-defines.h>
@@ -24,6 +25,8 @@
 #include <glib/gfileutils.h>
 #include <pango/pango.h>
 
+#include <hildon-extras/he-helper.h>
+
 #include "appdata.h"
 #include "interface.h"
 #include "callbacks.h"
@@ -149,51 +152,9 @@ void load_prev_image(GtkWidget *callerobj,app_data_t *myapp) {
 	}
 }
 
-/*
- * Draw image details
- */
-gboolean draw_image_details(GtkWidget *self,GdkEventExpose *event,app_data_t *myapp) {
-	PangoLayout *textbuff;
-	cairo_t *cr;
-	gint x0, y0, r, txtwidth, txtheight;
-
-	/* Get the Cairo context */
-	cr = gdk_cairo_create(GDK_DRAWABLE (self->window));
-	/* Clear the surface first */
-	cairo_set_operator(cr,CAIRO_OPERATOR_CLEAR);
-	cairo_paint(cr);
-	/* Create the text layout */
-	textbuff = pango_cairo_create_layout(cr);
-	pango_layout_set_markup(textbuff,myapp->imgparam,-1);
-	pango_layout_get_pixel_size(textbuff,&txtwidth,&txtheight);
-	/* Draw a rounded rectangle */
-	x0 = y0 = r = 10;
-	cairo_set_operator(cr,CAIRO_OPERATOR_OVER);
-	cairo_set_source_rgba(cr,0,0,0,0.4);
-	cairo_move_to(cr,x0,y0 - r);
-	cairo_line_to(cr,x0 + txtwidth,y0 -r);
-	cairo_curve_to(cr,x0 + txtwidth + r,y0 -r,x0 + txtwidth + r,y0 -r,x0 + txtwidth + r,y0);
-	cairo_line_to(cr,x0 + txtwidth + r,y0 + txtheight);
-	cairo_curve_to(cr,x0 + txtwidth + r,y0 + txtheight + r,x0 + txtwidth + r,y0 + txtheight + r,x0 + txtwidth,y0 + txtheight + r);
-	cairo_line_to(cr,x0,y0 + txtheight + r);
-	cairo_curve_to(cr,x0 -r,y0 + txtheight + r,x0 -r,y0 + txtheight + r,x0 -r,y0 + txtheight);
-	cairo_line_to(cr,x0 - r,y0);
-	cairo_curve_to(cr,x0 - r,y0 -r,x0 -r,y0 -r,x0,y0 - r);
-	cairo_close_path(cr);
-	cairo_fill(cr);
-	/* Layout the text on it */
-	cairo_new_path(cr);
-	cairo_move_to(cr,x0,y0);
-	cairo_set_line_width(cr,1);
-	cairo_set_source_rgb(cr,0.8,0.8,0.8);
-	pango_cairo_update_layout(cr,textbuff);
-	pango_cairo_layout_path(cr,textbuff);
-	cairo_fill_preserve(cr);
-	/*cairo_set_line_width(cr,0.5);
-	cairo_set_source_rgb(cr,0,0,0);
-	cairo_stroke(cr);*/
-	cairo_destroy(cr);
-	g_object_unref(textbuff);
+gboolean on_is_topmost_changed(GObject *object G_GNUC_UNUSED,GParamSpec *property G_GNUC_UNUSED,app_data_t *myapp) {
+	if (!hildon_window_get_is_topmost(myapp->mainwin)) he_helper_accelerometer_disable(myapp->ossocont);
+	else he_helper_accelerometer_enable(myapp->ossocont);
 
 	return FALSE;
 }
@@ -212,6 +173,29 @@ void toggle_prev_next_menu_button(const gchar *mode,app_data_t *myapp) {
 	}
 }
 
+/*
+ * Toggle image details window on or off
+ */
+void toggle_image_details(GtkWidget *callerobj,app_data_t *myapp) {
+	GList *menuitems = NULL;
+
+	menuitems = hildon_app_menu_get_items(hildon_window_get_app_menu(myapp->mainwin));
+	if (menuitems != NULL) {
+		if (!myapp->showimgparam) {
+			myapp->showimgparam = TRUE;
+			gtk_button_set_label(GTK_BUTTON (g_list_nth_data(menuitems,3)),"Hide details");
+			g_object_set(G_OBJECT (myapp->imgparamwin),"enabled",TRUE,NULL);
+			imgparam_window_show(IMGPARAM_WINDOW (myapp->imgparamwin));
+		} else {
+			myapp->showimgparam = FALSE;
+			gtk_button_set_label(GTK_BUTTON (g_list_nth_data(menuitems,3)),"View details");
+			g_object_set(G_OBJECT (myapp->imgparamwin),"enabled",FALSE,NULL);
+			imgparam_window_hide(IMGPARAM_WINDOW (myapp->imgparamwin));
+		}
+		g_list_free(menuitems);
+	}
+}
+
 /*
  * Toggle main window's fullscreen status on or off
  */
@@ -231,62 +215,13 @@ void toggle_fullscreen_mode(GtkWidget *callerobj,app_data_t *myapp) {
 	}
 }
 
-/*
- * Manage button presses on image to load previous / next image in the same directory
- */
-gboolean detect_button_press_on_image (GtkWidget *eventbox,GdkEventButton *event,app_data_t *myapp) {
-	gint dispwidth;
-	action_t act;
-	glong butprtime;
-	gdouble butprx, butpry;
-	GTimeVal now;
-
-	if (g_list_length(myapp->allimgindir) > 1) {
-		g_get_current_time(&now);
-		/* DEBUG */
-		/* g_print ("Event box clicked at coordinates (%f,%f) at time %ld\n",event->x_root,event->y_root,now.tv_sec); */
-		switch (event->type) {
-			case GDK_BUTTON_PRESS:
-				myapp->pressbuttontime = now.tv_sec;
-				myapp->pressbuttonx = event->x_root;
-				myapp->pressbuttony = event->y_root;
-				dispwidth = gdk_screen_get_width(GTK_WINDOW (myapp->mainwin)->screen);
-				if (event->x_root >= ((gdouble) dispwidth / (gdouble) 2)) myapp->actioninprog = NEXT;
-				else myapp->actioninprog = PREVIOUS;
-				g_timeout_add(100,(GSourceFunc) draw_prev_next_arrow,(gpointer) myapp);
-				return TRUE;
-			case GDK_BUTTON_RELEASE:
-				/* Store actual values */
-				act = myapp->actioninprog;
-				butprtime = myapp->pressbuttontime;
-				butprx = myapp->pressbuttonx;
-				butpry = myapp->pressbuttony;
-				/* Reset values */
-				myapp->pressbuttontime = 0;
-				myapp->pressbuttonx = 0;
-				myapp->pressbuttony = 0;
-				myapp->actioninprog = NONE;
-				if (((now.tv_sec - butprtime) >= MINBUTPRESSTIME) &
-					(abs(round(event->x_root - butprx)) <= MAXBUTPRESSDISTANCE) &
-					(abs(round(event->y_root - butpry)) <= MAXBUTPRESSDISTANCE))
-				{
-					if (act == NEXT) load_next_image(eventbox,myapp);
-					else load_prev_image(eventbox,myapp);
-				} else g_source_remove_by_user_data((gpointer) myapp);
-				return TRUE;
-			default:
-				break;
-		}
-	}
-	return FALSE;
-}
-
 /*
  * Manage key presses
  */
 gboolean detect_hardware_key_press (GtkWidget *widget,GdkEventKey *event,app_data_t *myapp) {
-	/* DEBUG */
-	/* g_print ("Detected key press: %d\n",event->keyval); */
+	guint scaledwidth, scaledheight;
+	zoom_t tmpzl;
+
 	switch (event->keyval) {
 		case HILDON_HARDKEY_FULLSCREEN:
 			toggle_fullscreen_mode(widget,myapp);
@@ -304,12 +239,13 @@ gboolean detect_hardware_key_press (GtkWidget *widget,GdkEventKey *event,app_dat
 			load_prev_image(widget,myapp);
 			return TRUE;
 		case HILDON_HARDKEY_INCREASE:
-			myapp->zoomlevel = (myapp->zoomlevel + 1) % 4;
-			scale_image(widget,myapp);
-			return TRUE;
 		case HILDON_HARDKEY_DECREASE:
-			myapp->zoomlevel = (myapp->zoomlevel + 3) % 4;
-			scale_image(widget,myapp);
+			tmpzl = myapp->zoomlevel;
+			do {
+				myapp->zoomlevel = (myapp->zoomlevel + ((event->keyval == HILDON_HARDKEY_INCREASE) ? 1 : 3)) % 4;
+				scaled_image_compute_size(myapp,&scaledwidth,&scaledheight);
+			} while ((myapp->zoomlevel != tmpzl) && (myapp->zoomlevel != FULL_RESOLUTION) && (scaledwidth == 0) && (scaledheight == 0));
+			if (myapp->zoomlevel != tmpzl) scale_image(widget,myapp);
 			return TRUE;
 		default:
 			break;
@@ -320,18 +256,8 @@ gboolean detect_hardware_key_press (GtkWidget *widget,GdkEventKey *event,app_dat
 /*
  * Manage main window size changes
  */
-gboolean detect_main_window_changes (GObject *object,GdkEvent *event,app_data_t *myapp) {
-	/* DEBUG */
-	/* g_print ("Detected event: %d\n",event->type); */
-	switch (event->type) {
-		case GDK_CONFIGURE:
-			scale_image(GTK_WIDGET (myapp->mainwin),myapp);
-			break;
-		case GDK_EXPOSE:
-		default:
-			break;
-	}
-	return FALSE;
+void detect_main_window_changes (GObject *object,GtkAllocation *allocation,app_data_t *myapp) {
+	scale_image(GTK_WIDGET (myapp->mainwin),myapp);
 }
 
 /*
@@ -352,16 +278,46 @@ DBusHandlerResult detect_mce_signal (DBusConnection *conn,DBusMessage *msg,app_d
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
+/*
+ * Detect session D-Bus messages
+ */
+gint detect_dbus_message (const gchar *interface,const gchar *method,GArray *arguments,app_data_t *myapp,osso_rpc_t *retval) {
+    osso_rpc_t *tmpval;
+    gchar *tmpfname;
+
+    if ((g_strncasecmp(method,"mime_open",9) == 0) & (arguments->len == 1)) {
+    	/* Load requested image */
+    	tmpval = &g_array_index(arguments,osso_rpc_t,0);
+    	if (tmpval->type == DBUS_TYPE_STRING) {
+			tmpfname = tmpval->value.s;
+    		if (g_str_has_prefix(tmpval->value.s,"file://")) tmpfname += strlen("file://");
+    		g_free(myapp->currdir);
+    		g_free(myapp->currfname);
+    		myapp->currdir = g_strndup(g_path_get_dirname(tmpfname),MAXDIRLEN);
+    		myapp->currfname = g_strndup(g_path_get_basename(tmpfname),MAXFNLEN);
+    		store_filenames_in_dir(myapp);
+    		load_image(myapp,g_strconcat(myapp->currdir,"/",myapp->currfname,NULL));
+    	} else return OSSO_ERROR;
+    } else if (g_strncasecmp(method,"top_application",15) == 0) {
+        /* Either show the "Open ..." dialog if there's no filename provided at startup time or automatically load last image opened */
+        if (!myapp->currfname || !g_file_test(g_strconcat(myapp->currdir,"/",myapp->currfname,NULL),G_FILE_TEST_EXISTS)) open_image_file(GTK_WIDGET (myapp->mainwin),myapp);
+        else load_image(myapp,g_strconcat(myapp->currdir,"/",myapp->currfname,NULL));
+    }
+	osso_rpc_free_val(retval);
+    return OSSO_OK;
+}
+
 /*
  * Quit application
  */
 void destroy_app (GtkWidget *callerobj,app_data_t *myapp) {
 	/* Save configuration */
 	save_configuration(myapp);
-	/* Quit GTK main loop */
+	/* Destroy widgets and quit GTK main loop */
+	gtk_widget_destroy(GTK_WIDGET (myapp->mainwin));
 	gtk_main_quit();
 	/* Disable accelerometers */
-	enable_accelerometers(myapp,FALSE);
+	he_helper_accelerometer_disable(myapp->ossocont);
 	/* Deinitialize OSSO context */
     osso_deinitialize(myapp->ossocont);
     /* Release memory allocated for application's data */
