@@ -1,14 +1,3 @@
-/*
- * Constants definitions
- */
-#define MAXDIRLEN 512
-#define MAXFNLEN 128
-#define MINBUTPRESSTIME 1
-#define MAXBUTPRESSDISTANCE 10
-#define MCE_SIGNAL_MATCH "type='signal'," \
-        "sender='"    MCE_SERVICE     "'," \
-        "path='"      MCE_SIGNAL_PATH "'," \
-        "interface='" MCE_SIGNAL_IF   "'"
 /*
  * Functions declaration
  */
@@ -17,9 +6,10 @@ void load_next_image(GtkWidget *callerobj,app_data_t *myapp);
 void load_prev_image(GtkWidget *callerobj,app_data_t *myapp);
 void toggle_prev_next_menu_button(const gchar *mode,app_data_t *myapp);
 void toggle_fullscreen_mode(GtkWidget *callerobj,app_data_t *myapp);
-gboolean detect_button_press_on_image (GtkWidget *eventbox,GdkEventButton *event,app_data_t *myapp);
+void toggle_image_details(GtkWidget *callerobj,app_data_t *myapp);
 gboolean detect_hardware_key_press (GtkWidget *widget,GdkEventKey *event,app_data_t *myapp);
-gboolean detect_main_window_changes (GObject *object,GdkEvent *event,app_data_t *myapp);
+void detect_main_window_changes (GObject *object,GtkAllocation *allocation,app_data_t *myapp);
 DBusHandlerResult detect_mce_signal (DBusConnection *conn,DBusMessage *msg,app_data_t *myapp);
-gboolean draw_image_details(GtkWidget *self,GdkEventExpose *event,app_data_t *myapp);
+gint detect_dbus_message (const gchar *interface,const gchar *method,GArray *arguments,app_data_t *myapp,osso_rpc_t *retval);
 void destroy_app (GtkWidget *callerobj,app_data_t *myapp);
+gboolean on_is_topmost_changed(GObject *object,GParamSpec *property,app_data_t *myapp);
