@@ -12,9 +12,6 @@
 #include <math.h>
 #include <glob.h>
 
-#include <dbus/dbus.h>
-#include <mce/mode-names.h>
-#include <mce/dbus-names.h>
 #include <libosso.h>
 #include <hildon/hildon.h>
 #include <hildon/hildon-defines.h>
@@ -40,44 +37,79 @@
  * Load image from filename
  */
 void load_image(app_data_t *myapp, gchar *imgfname) {
-	GtkWidget *errdialog;
-	GdkPixbufLoader *imgloader;
-	libraw_data_t *rawdata;
-	gboolean imgerr = FALSE;
+	GtkWidget *errdialog = NULL;
+	GdkPixbufLoader *imgloader = NULL;
+	GdkPixbuf *tmpimgbuffer = NULL;
+	libraw_data_t *rawdata = NULL;
+	libraw_processed_image_t *imgdata = NULL;
+	int rescode = LIBRAW_UNSPECIFIED_ERROR;
+	gboolean imgerr = FALSE, imgthumb = TRUE;
 	gint shutternum = 0;
-	gchar *shutter;
+	gchar *shutter = NULL;
 	const gchar *paramformat = "<span size=\"medium\" face=\"arial\">Filename: <b>%s</b>\nCamera model: <b>%s %s</b>\nShutter speed: <b>%s s</b>\nAperture: <b>f/%-.1f</b>\nISO: <b>%-.0f</b>\nFocal length: <b>%-.0f mm</b></span>";
+	GError *error = NULL;
 
 	g_free(myapp->currfname);
 	myapp->currfname = g_strndup(g_path_get_basename(imgfname),MAXFNLEN);
 	gtk_window_set_title(GTK_WINDOW (myapp->mainwin),g_strconcat(g_get_application_name()," - ",myapp->currfname,NULL));
 	hildon_gtk_window_set_progress_indicator(GTK_WINDOW (myapp->mainwin),1);
+	if (gdk_window_get_state(gtk_widget_get_window(GTK_WIDGET (myapp->mainwin))) & GDK_WINDOW_STATE_FULLSCREEN) hildon_banner_show_informationf(GTK_WIDGET (myapp->mainwin),NULL,"Please wait while loading image:\n%s",myapp->currfname);
 	while (gtk_events_pending()) gtk_main_iteration();
 	rawdata = libraw_init(0);
+	rawdata->params.use_camera_wb = 1;
 	/* DEBUG */
 	/* g_print("Supported RAW formats: %s\n",g_strjoinv("\n",(gchar**) libraw_cameraList())); */
-	if (libraw_open_file(rawdata,imgfname) == LIBRAW_SUCCESS) {
-		if (libraw_unpack_thumb(rawdata) == LIBRAW_SUCCESS) {
+	rescode = libraw_open_file(rawdata,imgfname);
+	if (rescode == LIBRAW_SUCCESS) {
+		/* First try to retrieve embedded thumbnail */
+		rescode = libraw_unpack_thumb(rawdata);
+		if (rescode == LIBRAW_SUCCESS) {
 			imgloader = gdk_pixbuf_loader_new();
-			if (gdk_pixbuf_loader_write(imgloader,(guchar *) rawdata->thumbnail.thumb,(gsize) rawdata->thumbnail.tlength,NULL)) {
+			if (!gdk_pixbuf_loader_write(imgloader,(guchar *) rawdata->thumbnail.thumb,(gsize) rawdata->thumbnail.tlength,&error)) {
+				imgerr = TRUE;
+				rescode = LIBRAW_UNSPECIFIED_ERROR;
+			} else tmpimgbuffer = gdk_pixbuf_loader_get_pixbuf(imgloader);
+			gdk_pixbuf_loader_close(imgloader,NULL);
+		} else if ((rescode == LIBRAW_NO_THUMBNAIL) | (rescode = LIBRAW_UNSUPPORTED_THUMBNAIL)) {
+			/* Then try to process RAW data */
+			imgthumb = FALSE;
+			rescode = libraw_unpack(rawdata);
+			if (rescode == LIBRAW_SUCCESS) {
+				rescode = libraw_dcraw_process(rawdata);
+				if (rescode == LIBRAW_SUCCESS) {
+					imgdata = libraw_dcraw_make_mem_image(rawdata,&rescode);
+					if ((rescode == LIBRAW_SUCCESS) & (imgdata != NULL)) {
+						if ((imgdata->type != LIBRAW_IMAGE_BITMAP) | !(tmpimgbuffer = gdk_pixbuf_new_from_data((guchar *) imgdata->data,GDK_COLORSPACE_RGB,FALSE,imgdata->bits,(int) imgdata->width,(int) imgdata->height,(imgdata->data_size / imgdata->height),NULL,NULL))) {
+							imgerr = TRUE;
+							rescode = LIBRAW_UNSPECIFIED_ERROR;
+						}
+					} else imgerr = TRUE;
+				} else imgerr = TRUE;
+			} else imgerr = TRUE;
+		} else imgerr = TRUE;
+		if (!imgerr) {
+			if (imgthumb) {
 				switch(rawdata->sizes.flip) {
 					case 3:
-						myapp->imgbuffer = gdk_pixbuf_rotate_simple(gdk_pixbuf_loader_get_pixbuf(imgloader),GDK_PIXBUF_ROTATE_UPSIDEDOWN);
+						myapp->imgbuffer = gdk_pixbuf_rotate_simple(tmpimgbuffer,GDK_PIXBUF_ROTATE_UPSIDEDOWN);
 						break;
 					case 5:
-						myapp->imgbuffer = gdk_pixbuf_rotate_simple(gdk_pixbuf_loader_get_pixbuf(imgloader),GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
+						myapp->imgbuffer = gdk_pixbuf_rotate_simple(tmpimgbuffer,GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
 						break;
 					case 6:
-						myapp->imgbuffer = gdk_pixbuf_rotate_simple(gdk_pixbuf_loader_get_pixbuf(imgloader),GDK_PIXBUF_ROTATE_CLOCKWISE);
+						myapp->imgbuffer = gdk_pixbuf_rotate_simple(tmpimgbuffer,GDK_PIXBUF_ROTATE_CLOCKWISE);
 						break;
 					case 0:
 					default:
-						myapp->imgbuffer = gdk_pixbuf_loader_get_pixbuf(imgloader);
+						myapp->imgbuffer = gdk_pixbuf_copy(tmpimgbuffer);
 						break;
 				}
-			} else imgerr = TRUE;
-			gdk_pixbuf_loader_close(imgloader,NULL);
-		} else imgerr = TRUE;
+			} else {
+				myapp->imgbuffer = gdk_pixbuf_copy(tmpimgbuffer);
+			}
+			g_object_unref(tmpimgbuffer);
+			if (imgdata) g_free(imgdata);
+		}
 	} else imgerr = TRUE;
 	while (gtk_events_pending()) gtk_main_iteration();
 	hildon_gtk_window_set_progress_indicator(GTK_WINDOW (myapp->mainwin),0);
@@ -87,173 +119,118 @@ void load_image(app_data_t *myapp, gchar *imgfname) {
 		shutter = g_strdup_printf(((rawdata->other.shutter < 1) ? "1/%d" : "%d"),shutternum);
 		myapp->imgparam = g_strdup_printf(paramformat,myapp->currfname,rawdata->idata.make,rawdata->idata.model,shutter,rawdata->other.aperture,rawdata->other.iso_speed,rawdata->other.focal_len);
 		g_free(shutter);
+		myapp->zoomlevel = FIT_SCREEN;
 		scale_image(GTK_WIDGET (myapp->mainwin),myapp);
+		g_object_set(G_OBJECT (myapp->imgparamwin),"markup",myapp->imgparam,NULL);
 	} else {
 		g_object_unref(myapp->imgbuffer);
 		gtk_image_set_from_pixbuf(GTK_IMAGE (myapp->image),NULL);
+		g_object_set(G_OBJECT (myapp->imgparamwin),"markup",NULL,NULL);
+		imgparam_window_hide(IMGPARAM_WINDOW (myapp->imgparamwin));
 		errdialog = gtk_message_dialog_new(GTK_WINDOW (myapp->mainwin),
 											GTK_DIALOG_DESTROY_WITH_PARENT,
 											GTK_MESSAGE_ERROR,
 											GTK_BUTTONS_CLOSE,
-											"Error loading image '%s'",
-											g_path_get_basename(imgfname));
-		 gtk_dialog_run(GTK_DIALOG (errdialog));
-		 gtk_widget_destroy(errdialog);
+											"There was an error while loading \"%s\"\n\nError message: \"%s\"",
+											g_path_get_basename(imgfname),
+											libraw_strerror(rescode));
+		gtk_window_set_title(GTK_WINDOW (errdialog),"Error loading image");
+		gtk_dialog_run(GTK_DIALOG (errdialog));
+		gtk_widget_destroy(errdialog);
 	}
 	libraw_close(rawdata);
 }
 
+/*
+ * Compute dimensions of scaled image according to selected zoom factor
+ */
+void scaled_image_compute_size(app_data_t *myapp,guint *scaledwidth,guint *scaledheight) {
+	gint dispwidth, dispheight, imgwidth, imgheight;
+	gdouble dispratio, imgratio;
+
+	/* Get actual image widget dimensions */
+	gtk_window_get_size(GTK_WINDOW (myapp->mainwin),&dispwidth,&dispheight);
+	dispratio = (gdouble) dispwidth / (gdouble) dispheight;
+	imgwidth = gdk_pixbuf_get_width(myapp->imgbuffer);
+	imgheight = gdk_pixbuf_get_height(myapp->imgbuffer);
+	imgratio = (gdouble) imgwidth / (gdouble) imgheight;
+	*scaledwidth = *scaledheight = 0;
+	/* Compute scaled image size */
+	if ((imgwidth > dispwidth) || (imgheight > dispheight)) {
+		switch (myapp->zoomlevel) {
+			case FIT_SCREEN:
+				if (imgratio >= dispratio) {
+					*scaledwidth = dispwidth;
+					*scaledheight = (guint) round((double) dispwidth * (1 / imgratio));
+				} else {
+					*scaledwidth = (guint) round((double) (dispheight * imgratio));
+					*scaledheight = dispheight;
+				}
+				break;
+			case ONE_THIRD_RESOLUTION:
+				*scaledwidth = (guint) round((double) imgwidth * 0.33);
+				*scaledheight = (guint) round((double) imgheight * 0.33);
+				break;
+			case HALF_RESOLUTION:
+				*scaledwidth = (guint) round((double) imgwidth * 0.5);
+				*scaledheight = (guint) round((double) imgheight * 0.5);
+				break;
+			case FULL_RESOLUTION:
+			default:
+				break;
+		}
+	}
+	/* DEBUG */
+	/* g_print("\n\nOriginal dim.: %dx%d (%f) - Window dim.: %dx%d (%f) - Scaled dim.: %dx%d\n\n",imgwidth,imgheight,imgratio,dispwidth,dispheight,dispratio,*scaledwidth,*scaledheight); */
+}
+
 /*
  * Scale current image according to selected zoom factor
  */
 gboolean scale_image(GtkWidget *callerobj,app_data_t *myapp) {
 	const gchar *zoomtext;
 	GdkPixbuf *tmpbuffer = NULL;
-	gint dispwidth, dispheight, imgwidth, imgheight;
-	guint scaledwidth = 0, scaledheight = 0;
-	gdouble dispratio, imgratio;
+	guint scaledwidth, scaledheight;
 	GdkInterpType scalingmode = GDK_INTERP_BILINEAR;
 
 	/* DEBUG */
 	/* g_print("Scale image with buffer: %d (%d)\n",myapp->imgbuffer,G_IS_OBJECT (G_OBJECT (myapp->imgbuffer))); */
 	if (myapp->imgbuffer != NULL) {
+		switch (myapp->zoomlevel) {
+			case FIT_SCREEN:
+				/* scalingmode = GDK_INTERP_NEAREST; */
+				zoomtext = " (Fit Scr.)";
+				break;
+			case ONE_THIRD_RESOLUTION:
+				zoomtext = " (33%)";
+				break;
+			case HALF_RESOLUTION:
+				zoomtext = " (50%)";
+				break;
+			case FULL_RESOLUTION:
+				zoomtext = " (Full Res.)";
+				break;
+			default:
+				zoomtext = "";
+				break;
+		}
 		hildon_gtk_window_set_progress_indicator(GTK_WINDOW (myapp->mainwin),1);
+		if (gdk_window_get_state(gtk_widget_get_window(GTK_WIDGET (myapp->mainwin))) & GDK_WINDOW_STATE_FULLSCREEN) hildon_banner_show_informationf(GTK_WIDGET (myapp->mainwin),NULL,"Scaling image %s",zoomtext);
 		while (gtk_events_pending()) gtk_main_iteration();
-		/* Get actual image widget dimensions */
-		gtk_window_get_size(GTK_WINDOW (myapp->mainwin),&dispwidth,&dispheight);
-		dispratio = (gdouble) dispwidth / (gdouble) dispheight;
-		imgwidth = gdk_pixbuf_get_width(myapp->imgbuffer);
-		imgheight = gdk_pixbuf_get_height(myapp->imgbuffer);
-		imgratio = (gdouble) imgwidth / (gdouble) imgheight;
-		scaledwidth = scaledheight = 0;
 		/* Eventually scale image */
-		if ((imgwidth > dispwidth) || (imgheight > dispheight)) {
-			switch (myapp->zoomlevel) {
-				case FIT_SCREEN:
-					if (imgratio >= dispratio) {
-						scaledwidth = dispwidth;
-						scaledheight = (guint) round((double) dispwidth * (1 / imgratio));
-					} else {
-						scaledwidth = (guint) round((double) (dispheight * imgratio));
-						scaledheight = dispheight;
-					}
-					/* scalingmode = GDK_INTERP_NEAREST; */
-					zoomtext = "";
-					break;
-				case ONE_THIRD_RESOLUTION:
-					scaledwidth = (guint) round((double) imgwidth * 0.33);
-					scaledheight = (guint) round((double) imgheight * 0.33);
-					zoomtext = " (33%)";
-					break;
-				case HALF_RESOLUTION:
-					scaledwidth = (guint) round((double) imgwidth * 0.5);
-					scaledheight = (guint) round((double) imgheight * 0.5);
-					zoomtext = " (50%)";
-					break;
-				case FULL_RESOLUTION:
-				default:
-					zoomtext = " (Full Res.)";
-					break;
-			}
-			if ((scaledwidth > 0) & (scaledheight > 0)) {
-				tmpbuffer = gdk_pixbuf_scale_simple(myapp->imgbuffer,scaledwidth,scaledheight,scalingmode);
-			}
-		} else zoomtext = "";
+		scaled_image_compute_size(myapp,&scaledwidth,&scaledheight);
+		if ((scaledwidth > 0) & (scaledheight > 0)) {
+			tmpbuffer = gdk_pixbuf_scale_simple(myapp->imgbuffer,scaledwidth,scaledheight,scalingmode);
+		}
 		gtk_image_set_from_pixbuf(GTK_IMAGE (myapp->image), (tmpbuffer ? tmpbuffer : myapp->imgbuffer));
 		while (gtk_events_pending()) gtk_main_iteration();
 		hildon_gtk_window_set_progress_indicator(GTK_WINDOW (myapp->mainwin),0);
 		gtk_window_set_title(GTK_WINDOW (myapp->mainwin),g_strconcat(g_get_application_name()," - ",myapp->currfname,zoomtext,NULL));
 		if (tmpbuffer) g_object_unref(tmpbuffer);
-		/* DEBUG */
-		/* g_print("\n\nOriginal dim.: %dx%d (%f) - Window dim.: %dx%d (%f) - Scaled dim.: %dx%d\n\n",imgwidth,imgheight,imgratio,dispwidth,dispheight,dispratio,scaledwidth,scaledheight); */
 	}
 	return TRUE;
 }
 
-/*
- * Draw left / right arrows
- */
-gboolean draw_prev_next_arrow(app_data_t *myapp) {
-	GtkImageType imgtype;
-	cairo_t *cr;
-	GdkPixmap *tmppm;
-	GdkBitmap *tmpbm;
-	gint winwidth, winheight, pointerx, pointery, x0, y0, r, m, w, h;
-	GTimeVal now;
-
-	g_get_current_time(&now);
-	gdk_window_get_pointer(gtk_widget_get_root_window(GTK_WIDGET (myapp->mainwin)),&pointerx,&pointery,NULL);
-	if (myapp->actioninprog != NONE) {
-		if (((now.tv_sec - myapp->pressbuttontime) >= MINBUTPRESSTIME) &
-			(abs(round((double)(pointerx - myapp->pressbuttonx))) <= MAXBUTPRESSDISTANCE) &
-			(abs(round((double)(pointery - myapp->pressbuttony))) <= MAXBUTPRESSDISTANCE))
-		{
-			imgtype = gtk_image_get_storage_type(GTK_IMAGE (myapp->image));
-			if (imgtype != GTK_IMAGE_EMPTY) {
-				/* If the image is zoomed in, zoom out to FIT_SCREEN before drawing the arrow */
-				if (myapp->zoomlevel != FIT_SCREEN) {
-					myapp->zoomlevel = FIT_SCREEN;
-					scale_image(GTK_WIDGET (myapp->mainwin),myapp);
-				}
-				if (imgtype == GTK_IMAGE_PIXBUF) gdk_pixbuf_render_pixmap_and_mask(gtk_image_get_pixbuf(GTK_IMAGE (myapp->image)),&tmppm,&tmpbm,0);
-				else if (imgtype == GTK_IMAGE_PIXMAP) {
-					gtk_image_get_pixmap(GTK_IMAGE (myapp->image),&tmppm,&tmpbm);
-					tmppm = g_object_ref(tmppm);
-					tmpbm = g_object_ref(tmpbm);
-				} else return FALSE;
-				gdk_drawable_get_size(GDK_DRAWABLE (tmppm),&winwidth,&winheight);
-				r = 5;
-				m = 15;
-				w = h = 40;
-				if (myapp->actioninprog == NEXT) x0 = (winwidth - w - m);
-				else x0 = m;
-				y0 = round((double)((winheight - h) / 2));
-				cr = gdk_cairo_create(GDK_DRAWABLE (tmppm));
-
-				/* Draw a rounded rectangle */
-				cairo_set_source_rgba(cr,0,0,0,0.4);
-				cairo_move_to(cr,x0,y0 - r);
-				cairo_line_to(cr,x0 + w,y0 -r);
-				cairo_curve_to(cr,x0 + w + r,y0 -r,x0 + w + r,y0 -r,x0 + w + r,y0);
-				cairo_line_to(cr,x0 + w + r,y0 + h);
-				cairo_curve_to(cr,x0 + w + r,y0 + h + r,x0 + w + r,y0 + h + r,x0 + w,y0 + h + r);
-				cairo_line_to(cr,x0,y0 + h + r);
-				cairo_curve_to(cr,x0 -r,y0 + h + r,x0 -r,y0 + h + r,x0 -r,y0 + h);
-				cairo_line_to(cr,x0 - r,y0);
-				cairo_curve_to(cr,x0 - r,y0 -r,x0 -r,y0 -r,x0,y0 - r);
-				cairo_close_path(cr);
-				cairo_fill(cr);
-				/* Draw an arrow according to the action being performed*/
-				cairo_new_path(cr);
-				cairo_set_source_rgba(cr,1,1,1,0.6);
-				if (myapp->actioninprog == NEXT) {
-					cairo_move_to(cr,x0,y0);
-					cairo_line_to(cr,x0 + w,y0 + (h / 2));
-					cairo_line_to(cr,x0,y0 + h);
-					cairo_line_to(cr,x0,y0);
-				} else {
-					cairo_move_to(cr,x0,y0 + (h / 2));
-					cairo_line_to(cr,x0 + w,y0);
-					cairo_line_to(cr,x0 + w,y0 + h);
-					cairo_line_to(cr,x0,y0 + (h / 2));
-				}
-				cairo_close_path(cr);
-				cairo_fill(cr);
-
-				cairo_destroy(cr);
-				gtk_image_set_from_pixmap(GTK_IMAGE (myapp->image),tmppm,tmpbm);
-				g_object_unref(tmppm);
-				/*
-				 * Un-referencing tmpbm is not needed
-				 * g_object_unref(tmpbm);
-				 *
-				 */
-			}
-			return FALSE;
-		} else return TRUE;
-	} else return FALSE;
-}
-
 /*
  * Store image file names in current directory into application's data structure
  */
@@ -286,7 +263,7 @@ void store_filenames_in_dir(app_data_t *myapp) {
 		g_free(tmpstr);
 	}
 	/* DEBUG */
-	/* for(li = myapp->allimgindir;li != NULL;li = g_list_next(li))
+	/*for(li = myapp->allimgindir;li != NULL;li = g_list_next(li))
 		g_print("File in list: %s\n",li->data);
 	g_list_free(li);*/
 }
@@ -327,6 +304,9 @@ void save_configuration(app_data_t *myapp) {
 		if (!gconf_client_set_int(gcClient,g_strconcat(GC_ROOT,"zoomlevel",NULL),myapp->zoomlevel,NULL)) {
 			g_printerr("WARNING: Failed to set %szoomlevel to %d\n",GC_ROOT,myapp->zoomlevel);
 		}
+		if (!gconf_client_set_bool(gcClient,g_strconcat(GC_ROOT,"showimgparam",NULL),myapp->showimgparam,NULL)) {
+			g_printerr("WARNING: Failed to set %sshowimgparam to %d\n",GC_ROOT,myapp->showimgparam);
+		}
 		g_object_unref(gcClient);
 	} else g_printerr("WARNING: Failed to get default GConf client.\n");
 }
@@ -344,16 +324,7 @@ void load_configuration(app_data_t *myapp) {
 		myapp->currfname = gconf_client_get_string(gcClient,g_strconcat(GC_ROOT,"currfname",NULL),NULL);
 		myapp->currfindex = gconf_client_get_int(gcClient,g_strconcat(GC_ROOT,"currfindex",NULL),NULL);
 		myapp->zoomlevel = gconf_client_get_int(gcClient,g_strconcat(GC_ROOT,"zoomlevel",NULL),NULL);
+		myapp->showimgparam = gconf_client_get_bool(gcClient,g_strconcat(GC_ROOT,"showimgparam",NULL),NULL);
 		g_object_unref(gcClient);
-		store_filenames_in_dir(myapp);
 	} else g_printerr("WARNING: Failed to get default GConf client.\n");
 }
-
-/*
- * Enable or disable accelerometers
- */
-void enable_accelerometers(app_data_t *myapp,gboolean mode) {
-	if (osso_rpc_run_system(myapp->ossocont,MCE_SERVICE,MCE_REQUEST_PATH,MCE_REQUEST_IF,
-			(mode ? MCE_ACCELEROMETER_ENABLE_REQ : MCE_ACCELEROMETER_DISABLE_REQ),NULL,DBUS_TYPE_INVALID) != OSSO_OK)
-		g_printerr("WARNING: Cannot %s accelerometers\n",(mode ? "enable" : "disable"));
-}
