@@ -20,13 +20,15 @@
 #include <gtk/gtk.h>
 #include <pango/pango.h>
 
+#include <hildon-extras/he-fullscreen-button.h>
+#include <hildon-extras/he-helper.h>
 #include <libraw/libraw.h>
 
 #include "appdata.h"
 #include "interface.h"
 #include "callbacks.h"
 #include "helpers.h"
-#include "he-fullscreen-button.h"
+#include "arrow-button.h"
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
 #endif
@@ -45,9 +47,11 @@ void create_app(app_data_t *myapp) {
 	GtkWidget *panarea;
     GtkWidget *eventbox;
     HeFullscreenButton *fsbutton;
+    ArrowButton *prevarbutton,*nextarbutton;
+    osso_return_t ossores;
 
     /* Enable accelerometers */
-    enable_accelerometers(myapp,TRUE);
+    he_helper_accelerometer_enable(myapp->ossocont);
 
     /* Create the hildon program and setup the title */
     myapp->app = HILDON_PROGRAM(hildon_program_get_instance());
@@ -77,8 +81,8 @@ void create_app(app_data_t *myapp) {
     g_signal_connect_after(G_OBJECT (openitem),"clicked",G_CALLBACK (open_image_file),myapp);
     hildon_app_menu_insert(appmenu,GTK_BUTTON(openitem),2);
 
-    imgdetitem = gtk_button_new_with_label("View details");
-    g_signal_connect_after(G_OBJECT (imgdetitem),"clicked",G_CALLBACK (create_image_details),myapp);
+    imgdetitem = gtk_button_new_with_label((myapp->showimgparam ? "Hide details" : "View details"));
+    g_signal_connect_after(G_OBJECT (imgdetitem),"clicked",G_CALLBACK (toggle_image_details),myapp);
     hildon_app_menu_insert(appmenu,GTK_BUTTON (imgdetitem),3);
 
     fullscreenitem = gtk_button_new_with_label("Go fullscreen");
@@ -109,12 +113,8 @@ void create_app(app_data_t *myapp) {
     /* Show everything */
     gtk_widget_show_all(GTK_WIDGET (myapp->mainwin));
 
-    /* Add Fullscreen button */
-    fsbutton = he_fullscreen_button_new(GTK_WINDOW (myapp->mainwin));
-    g_signal_connect(G_OBJECT (fsbutton),"clicked",G_CALLBACK (toggle_fullscreen_mode),myapp);
-
-    /* Load configuration from GConf */
-    load_configuration(myapp);
+    /* Save list of known image files in current directory */
+	store_filenames_in_dir(myapp);
 
     /* Ungrab volume keys */
     ungrab_volume_keys(GTK_WIDGET (myapp->mainwin));
@@ -122,25 +122,40 @@ void create_app(app_data_t *myapp) {
     /* Connect some signals */
     g_signal_connect(G_OBJECT (myapp->mainwin),"destroy",
     		G_CALLBACK (destroy_app),myapp);
-    g_signal_connect(G_OBJECT (myapp->mainwin),"configure_event",
+    g_signal_connect_after(G_OBJECT (myapp->mainwin),"size-allocate",
     		G_CALLBACK (detect_main_window_changes),myapp);
-    g_signal_connect(G_OBJECT (eventbox),"button_press_event",
-    		G_CALLBACK (detect_button_press_on_image),myapp);
-    g_signal_connect(G_OBJECT (eventbox),"button_release_event",
-        	G_CALLBACK (detect_button_press_on_image),myapp);
-    g_signal_connect(G_OBJECT(myapp->mainwin),"key_press_event",
+    g_signal_connect(G_OBJECT(myapp->mainwin),"key-press-event",
 			G_CALLBACK (detect_hardware_key_press),myapp);
+    g_signal_connect(G_OBJECT(myapp->mainwin),"notify::is-topmost",
+			G_CALLBACK (on_is_topmost_changed),myapp);
 
-    /* Connect DBus events handler */
-	myapp->sysbus = dbus_bus_get (DBUS_BUS_SYSTEM,NULL);
+    /* Connect system bus D-Bus events handler */
+	myapp->sysbus = dbus_bus_get(DBUS_BUS_SYSTEM,NULL);
 	if (myapp->sysbus) {
-		dbus_bus_add_match (myapp->sysbus,MCE_SIGNAL_MATCH,NULL);
+		dbus_bus_add_match(myapp->sysbus,MCE_SIGNAL_MATCH,NULL);
 		dbus_connection_add_filter(myapp->sysbus,(DBusHandleMessageFunction)detect_mce_signal,myapp,NULL);
 	}
 
-    /* Automatically show the "Open ..." dialog if there's no filename provided at start time or load last image opened */
-    if (!myapp->currfname || !g_file_test(g_strconcat(myapp->currdir,"/",myapp->currfname,NULL),G_FILE_TEST_EXISTS)) open_image_file(GTK_WIDGET (myapp->mainwin),myapp);
-    else load_image(myapp,g_strconcat(myapp->currdir,"/",myapp->currfname,NULL));
+	/* Connect session bus D-Bus events handler */
+	ossores = osso_rpc_set_cb_f(myapp->ossocont,APP_DBUS_SERVICE,APP_DBUS_OBJECT_PATH,APP_DBUS_INTERFACE,
+								(osso_rpc_cb_f *)detect_dbus_message,myapp);
+	if (ossores != OSSO_OK) {
+		g_print("Error setting session D-Bus default callback (%d)",ossores);
+	}
+
+    /* Add Fullscreen button */
+    fsbutton = he_fullscreen_button_new(GTK_WINDOW (myapp->mainwin));
+    g_signal_connect(G_OBJECT (fsbutton),"clicked",G_CALLBACK (toggle_fullscreen_mode),myapp);
+
+    /* Add previous / next arrow buttons */
+    prevarbutton = arrow_button_new(GTK_WINDOW (myapp->mainwin),ARROW_LEFT);
+    nextarbutton = arrow_button_new(GTK_WINDOW (myapp->mainwin),ARROW_RIGHT);
+    g_signal_connect(G_OBJECT (prevarbutton),"clicked",G_CALLBACK (load_prev_image),myapp);
+    g_signal_connect(G_OBJECT (nextarbutton),"clicked",G_CALLBACK (load_next_image),myapp);
+
+    /* Create image parameters window */
+    myapp->imgparamwin = imgparam_window_new(GTK_WINDOW (myapp->mainwin),NULL);
+    if (myapp->showimgparam) g_object_set(G_OBJECT (myapp->imgparamwin),"enabled",TRUE,NULL);
 
     /* Finally take a screenshot: say cheese :-) */
     hildon_gtk_window_take_screenshot(GTK_WINDOW (myapp->mainwin),TRUE);
@@ -188,50 +203,3 @@ void create_about(GtkWidget *callerobj,app_data_t *myapp) {
 	gtk_dialog_run(GTK_DIALOG (abdialog));
 	gtk_widget_destroy(abdialog);
 }
-
-/*
- * Create the image details widget
- */
-void create_image_details(GtkWidget *callerobj,app_data_t *myapp) {
-	GdkScreen *screen;
-	PangoLayout *textbuff;
-	cairo_t *cr;
-	gint offsetx, offsety, padding, txtwidth, txtheight;
-
-	/* For the widget itself let's use a GtkWindow */
-	if ((myapp->imgparamwin != NULL) & GTK_IS_WINDOW(myapp->imgparamwin)) draw_image_details(myapp->imgparamwin,NULL,myapp);
-	else {
-		myapp->imgparamwin = gtk_window_new(GTK_WINDOW_POPUP);
-		gtk_window_set_decorated(GTK_WINDOW (myapp->imgparamwin),FALSE);
-		gtk_widget_set_app_paintable(myapp->imgparamwin,TRUE);
-		screen = gtk_widget_get_screen(myapp->imgparamwin);
-		gtk_widget_set_colormap(myapp->imgparamwin,gdk_screen_get_rgba_colormap(screen));
-		gtk_window_set_transient_for(GTK_WINDOW (myapp->imgparamwin),GTK_WINDOW (myapp->mainwin));
-		gtk_window_set_destroy_with_parent(GTK_WINDOW (myapp->imgparamwin),TRUE);
-		gtk_window_set_modal(GTK_WINDOW (myapp->imgparamwin),TRUE);
-		gtk_widget_realize(myapp->imgparamwin);
-		/* Get the Cairo context and the overall dimensions of the text to be displayed */
-		cr = gdk_cairo_create(GDK_DRAWABLE (myapp->imgparamwin->window));
-		textbuff = pango_cairo_create_layout(cr);
-		pango_layout_set_markup(textbuff,myapp->imgparam,-1);
-		pango_layout_get_pixel_size(textbuff,&txtwidth,&txtheight);
-		cairo_destroy(cr);
-		g_object_unref(textbuff);
-		/* Show the widget */
-		padding = 20;
-		offsetx = (gint)round((double)(gdk_screen_get_width(screen) - (txtwidth + padding)) / 2);
-		offsety = (gint)round((double)(gdk_screen_get_height(screen) - (txtheight + padding)) / 2);
-		gtk_widget_set_size_request(myapp->imgparamwin,txtwidth + padding,txtheight + padding);
-		gtk_window_set_resizable(GTK_WINDOW (myapp->imgparamwin),FALSE);
-		/* gdk_window_set_override_redirect(myapp->imgparamwin->window,TRUE); */
-		/* gdk_window_reparent(myapp->imgparamwin->window,gtk_widget_get_window(GTK_WIDGET (myapp->mainwin)),offsetx,offsety); */
-		gtk_window_move(GTK_WINDOW (myapp->imgparamwin),offsetx,offsety);
-		gtk_grab_add(myapp->imgparamwin);
-		gtk_widget_show(myapp->imgparamwin);
-		/* Actual drawing needs to take place when the widget is exposed */
-		g_signal_connect_after(myapp->imgparamwin,"expose-event",
-				G_CALLBACK (draw_image_details),myapp);
-		g_signal_connect_after(myapp->imgparamwin,"button-press-event",
-				G_CALLBACK (gtk_widget_destroy),NULL);
-	}
-}
